Package: Rarr
Title: A Simple and Performant Native R Reader & Writer for Zarr Arrays
Version: 2.1.26
Authors@R: c(
    person("Mike", "Smith", role = c("aut", "ccp"),
           comment = c(ORCID = "0000-0002-7800-3848", "Maintainer from 2022 to 2025.")),
    person("Hugo", "Gruson", , "hugo.gruson@embl.de", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-4094-1476")),
    person("Artür", "Manukyan", role = "ctb"),
    person("Sharla", "Gelfand", role = "ctb"),
    person("German Network for Bioinformatics Infrastructure - de.NBI", role = "fnd",
           comment = c(ROR = "01vmpm840"))
  )
Description: The Zarr specification defines a format for chunked,
    compressed, N-dimensional arrays.  Its design allows efficient access
    to subsets of the stored array, and supports both local and cloud
    storage systems.  Rarr aims to implement this specification in R with
    minimal reliance on an external tools or libraries.
License: MIT + file LICENSE
URL: https://huber-group-embl.github.io/Rarr/,
        https://github.com/Huber-group-EMBL/Rarr
BugReports: https://github.com/Huber-group-EMBL/Rarr/issues
Depends: R (>= 4.2.0)
Imports: curl, grumpy, jsonlite, lifecycle, paws.storage, R.utils,
        rlang, utils
Suggests: BiocStyle, knitr, rmarkdown, testthat (>= 3.0.0), withr,
        ZarrArray
VignetteBuilder: knitr
biocViews: DataImport
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
SystemRequirements: GNU make
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
git_url: https://git.bioconductor.org/packages/Rarr
git_branch: devel
git_last_commit: 3f0925a
git_last_commit_date: 2026-07-19
Repository: Bioconductor 3.24
Date/Publication: 2026-07-20
NeedsCompilation: yes
Packaged: 2026-07-20 23:44:13 UTC; biocbuild
Author: Mike Smith [aut, ccp] (ORCID: <https://orcid.org/0000-0002-7800-3848>,
    Maintainer from 2022 to 2025.),
  Hugo Gruson [aut, cre] (ORCID: <https://orcid.org/0000-0002-4094-1476>),
  Artür Manukyan [ctb],
  Sharla Gelfand [ctb],
  German Network for Bioinformatics Infrastructure - de.NBI [fnd] (ROR:
    <https://ror.org/01vmpm840>)
Maintainer: Hugo Gruson <hugo.gruson@embl.de>
