# Authenticated S3 access works

    Code
      zarr_overview("https://s3.embl.de/rarr-testing/bzip2.zarr")
    Output
      Type: Array
      Path: https://s3.embl.de/rarr-testing/bzip2.zarr
      Shape: 20 x 10
      Chunk Shape: 10 x 10
      No. of Chunks: 2 (2 x 1)
      Data Type: int32
      Endianness: little
      Compressor: bz2

---

    Code
      read_zarr_array("https://s3.embl.de/rarr-testing/bzip2.zarr")
    Output
            [,1] [,2] [,3] [,4] [,5] [,6] [,7] [,8] [,9] [,10]
       [1,]    1    2    3    4    5    6    7    8    9    10
       [2,]    4    0    0    0    0    0    0    0    0     0
       [3,]    7    0    0    0    0    0    0    0    0     0
       [4,]   10    0    0    0    0    0    0    0    0     0
       [5,]   13    0    0    0    0    0    0    0    0     0
       [6,]   16    0    0    0    0    0    0    0    0     0
       [7,]   19    0    0    0    0    0    0    0    0     0
       [8,]   22    0    0    0    0    0    0    0    0     0
       [9,]   25    0    0    0    0    0    0    0    0     0
      [10,]   28    0    0    0    0    0    0    0    0     0
      [11,]   31    0    0    0    0    0    0    0    0     0
      [12,]   34    0    0    0    0    0    0    0    0     0
      [13,]   37    0    0    0    0    0    0    0    0     0
      [14,]   40    0    0    0    0    0    0    0    0     0
      [15,]   43    0    0    0    0    0    0    0    0     0
      [16,]   46    0    0    0    0    0    0    0    0     0
      [17,]   49    0    0    0    0    0    0    0    0     0
      [18,]   52    0    0    0    0    0    0    0    0     0
      [19,]   55    0    0    0    0    0    0    0    0     0
      [20,]   58    0    0    0    0    0    0    0    0     0

# JSON metadata is not unboxed

    Code
      zarr_overview(
        "https://livingobjects.ebi.ac.uk/idr/share/ome2024-ngff-challenge/idr0066/ExpA_VIP_ASLM_on_XZ_slice1088.zarr/0")
    Output
      Type: Array
      Path: https://livingobjects.ebi.ac.uk/idr/share/ome2024-ngff-challenge/idr0066/ExpA_VIP_ASLM_on_XZ_slice1088.zarr/0
      Shape: 1478 x 2048
      Chunk Shape: 1536 x 2048
      No. of Chunks: 1 (1 x 1)
      Data Type: uint16
      Endianness: NA
      Compressor: None
      Attributes: yes

