% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_metadata.R
\name{.read_array_metadata}
\alias{.read_array_metadata}
\title{Read the \code{.zarray} or \code{zarr.json} metadata file associated with a Zarr array}
\usage{
.read_array_metadata(zarr_path, s3_client = NULL)
}
\arguments{
\item{zarr_path}{A character vector of length 1.  This provides the
path to a Zarr array or group of arrays. This can either be on a local file
system or on S3 storage.}

\item{s3_client}{A list representing an S3 client.  This should be produced
by \code{\link[paws.storage:s3]{paws.storage::s3()}}.}
}
\value{
A list containing the array metadata
}
\description{
Read the \code{.zarray} or \code{zarr.json} metadata file associated with a Zarr array
}
\keyword{internal}
