% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/store_utils.R
\name{.read_json_file}
\alias{.read_json_file}
\title{Read a JSON file from local disk or S3}
\usage{
.read_json_file(path, s3_client = NULL)
}
\arguments{
\item{path}{Full path (local or S3) to a JSON file.}

\item{s3_client}{An S3 client produced by \code{\link[paws.storage:s3]{paws.storage::s3()}}, or \code{NULL}
for local files.}
}
\value{
A list as returned by \code{\link[jsonlite:read_json]{jsonlite::read_json()}} /
\code{\link[jsonlite:fromJSON]{jsonlite::fromJSON()}}.
}
\description{
Read a JSON file from local disk or S3
}
\keyword{internal}
