% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/codecs_utils.R
\name{.configure_codecs}
\alias{.configure_codecs}
\title{Convert the metadata \code{codecs} elements into functions}
\usage{
.configure_codecs(codecs, operation = c("encode", "decode"))
}
\arguments{
\item{codecs}{A list containing the Zarr v3 codecs}

\item{operation}{One of "encode" or "decode"}
}
\value{
An list of 3 environments containing functions:
\itemize{
\item \code{bytes_bytes_codecs}: functions to encode/decode raw bytes
\item \code{array_array_codecs}: functions to encode/decode R arrays
\item \code{array_bytes_codecs}: functions to encode/decode between R arrays and raw bytes
}
}
\description{
Convert the metadata \code{codecs} elements into functions
}
\keyword{internal}
