% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.extract_chunk}
\alias{.extract_chunk}
\title{Subset extraction for an array with a variable number of dimensions.}
\usage{
.extract_chunk(x, idx)
}
\arguments{
\item{x}{Array to extract from.}

\item{idx}{List of index vectors, one per dimension.}
}
\value{
The extracted sub-array (with \code{drop = FALSE}).
}
\description{
Subset extraction for an array with a variable number of dimensions.
}
\keyword{internal}
