% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rarr-deprecated.R
\name{ZarrArray}
\alias{ZarrArray}
\alias{writeZarrArray}
\title{Deprecated DelayedArray backend functions}
\usage{
ZarrArray(...)

writeZarrArray(...)
}
\arguments{
\item{...}{Passed to the new function in the ZarrArray package.}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#superseded}{\figure{lifecycle-superseded.svg}{options: alt='[Superseded]'}}}{\strong{[Superseded]}}

The DelayedArray backend has moved to a dedicated package: the ZarrArray package
(\url{https://github.com/Bioconductor/ZarrArray}).
}
