% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/s3.R
\name{.get_credentials}
\alias{.get_credentials}
\title{This is a modified version of paws.storage:::get_credentials().  It is
included to prevent using the \code{:::} operator.  Look at that function if
things stop working.}
\usage{
.get_credentials(credentials)
}
\arguments{
\item{credentials}{Content stored at \code{.internal$config$credentials} in
an object created by \code{paws.storage::s3()}.}
}
\value{
A credentials list to be reinserted into a \code{paws.storage} s3 object.
If no valid credentials are found this function will error, which is expected
and is caught by \code{.check_credentials}.
}
\description{
This is a modified version of paws.storage:::get_credentials().  It is
included to prevent using the \code{:::} operator.  Look at that function if
things stop working.
}
\keyword{internal}
