% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write_metadata.R
\name{write_zarr_group}
\alias{write_zarr_group}
\title{Initialize a Zarr group}
\usage{
write_zarr_group(
  zarr_path,
  group,
  zarr_version = if (has_metadata_v2) 2L else 3L
)
}
\arguments{
\item{zarr_path}{A character vector of length 1. This provides the
path to a Zarr store.}

\item{group}{A character vector of length 1. This provides the name of the
group to create. If \code{""}, the root group will be created.}

\item{zarr_version}{The version of the Zarr specification to use. If a
metadata file already exists, the version will be inferred from the file.
Otherwise, the default is \code{3}.}
}
\description{
Initialize a Zarr group
}
\details{
Nested groups are created recursively. For example, if
\code{group = "foo/bar"}, then the group \code{foo} will be created first,
followed by the group \code{bar} inside of it.
}
\examples{
zarr_v2 <- withr::local_tempfile(fileext = ".zarr")
write_zarr_group(zarr_v2, "test/deep/nested/group", zarr_version = 2L)

}
