% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write_data.R
\name{.compress_and_write_chunk}
\alias{.compress_and_write_chunk}
\title{Compress and write a single chunk}
\usage{
.compress_and_write_chunk(input_chunk, chunk_path, metadata)
}
\arguments{
\item{input_chunk}{Array containing the chunk data to be compressed.  Will be
converted to a raw vector before compression.}

\item{chunk_path}{Character string giving the path to the chunk that should
be written.}
}
\value{
Returns \code{TRUE} if writing is successful.  Mostly called for the
side-effect of writing the compressed chunk to disk.
}
\description{
Compress and write a single chunk
}
\keyword{internal}
