% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/store_utils.R
\name{.normalize_array_path}
\alias{.normalize_array_path}
\title{Normalize a Zarr array path}
\usage{
.normalize_array_path(path)
}
\arguments{
\item{path}{Character vector of length 1 giving the path to be normalised.}
}
\value{
A character vector of length 1 containing the normalised path.
}
\description{
Taken from https://zarr.readthedocs.io/en/stable/spec/v2.html#logical-storage-paths
}
\keyword{internal}
